"""Implement Bootstrap to compute histogram uncertainties.

Uncertainty is computed as Confidence Interval (CI).
Implementation of the percentile method [1, 2].

Basic assumptions:
- each data point (G') is independent and identically distributed

Ref
-------
[1] Bootstrap Methods and their applications, Davidson, 1997
[2] Introduction to bootstrap, Efron, 1993
"""
import numpy as np


def bootstrap_hist(data, bins, n_boots):
    """Compute bootstrap statistics.

    Parameters
    ----------
    data : array_like
        Data to be bootstrapped.
    statistics : Callable
        Statistics to compute.
    n_boots : int
        Number of bootstrap resamples.

    Returns
    -------
    array_like
        Bootstrapped statistics.
    """
    data = np.asarray(data)
    rng = np.random.default_rng()
    n = len(data)
    # rows: each resampling
    # columns: a statistics, corresponding to a bin height
    boots = np.empty((n_boots, len(bins) - 1))
    for i in range(n_boots):
        x, _ = np.histogram(rng.choice(data, size=n), bins=bins)
        boots[i] = x
    return boots


def statistics_ci(bootstrap_data, n_boots, original_data, alpha):
    """Compute confidence interval using percentile method."""
    # reorder bootstrap data to select extrema
    ci = np.zeros((original_data["bins"], 2))
    for idx in range(original_data["bins"]):
        # bootstrap data for each bin
        boostrap_data_i = bootstrap_data[:, idx].flatten()
        sorted_boot = np.sort(boostrap_data_i)
        i_l = round(alpha * n_boots) - 1  # -1 for 0-indexing
        i_u = round((1 - alpha) * n_boots) - 1
        ci[idx] = np.array([sorted_boot[i_l], sorted_boot[i_u]])

    return ci


if __file__ == "__main__":
    pass
