"""GE distribution for the N and the C terminal.
"""
import argparse
import csv
import pathlib

import matplotlib
import matplotlib.pyplot as plt
import numpy as np
import seaborn as sns
from bootstrap import bootstrap_hist, statistics_ci

from utils import fancy_grid, refined_text, remove_right_upper_axes, set_ticks

parent = pathlib.Path(__file__).parent.resolve()

# -------------- Parse arguments -------------- #
parser = argparse.ArgumentParser()
parser.add_argument(
    "region",
    help="Choose between 'domains' and 'chains'.",
    choices=["domains", "chains"],
)
parser.add_argument(
    "step",
    help="Bins step to use for the histogram.",
    type=float,
)
parser.add_argument(
    "n_boots",
    help="Number of bootstrap resampling.",
    type=int,
)
args = parser.parse_args()

# -------------- Load data -------------- #
parent = pathlib.Path(__file__).parent.resolve()
input_file = parent / "ge_list_files" / f"ge_c_n_{args.region}.csv"

# parse trans-mono info
trans_mono_dir = (
    parent / ".." /
    "pipeline/3_classification/classification/transmembrane_monotopic_list.csv"
)
with open(trans_mono_dir, "r") as in_file:
    reader = csv.reader(in_file, delimiter=",")
    transmono = {}
    next(reader)
    for row in reader:
        if not row[0] in transmono:
            transmono[row[0].replace("_", "").upper()] = row[1]

# colors for classification
group_trans = []
group_mono = []
group_not_specified = []

ge_n = []
ge_c = []
with open(input_file, "r") as in_file:
    reader = csv.reader(in_file, delimiter=",")
    for row in reader:
        if row[0].startswith("#"):
            continue

        if row[5] != "None" and row[10] != "None":
            ge_n.append(float(row[5]))
            ge_c.append(float(row[10]))
            if ge_n[-1] > 1.5 and ge_c[-1] < -1:
                print(row)
        else:
            continue

        if row[0][:5] in transmono:
            if transmono[row[0][:5]] == "trans":
                group_trans.append(1)
                group_mono.append(0)
                group_not_specified.append(0)
            elif transmono[row[0][:5]] == "mono":
                group_trans.append(0)
                group_mono.append(1)
                group_not_specified.append(0)
            elif transmono[row[0][:5]] == "other":
                group_trans.append(0)
                group_mono.append(0)
                group_not_specified.append(1)
        else:
            raise ValueError(f"Unknown classification: {transmono[row[0][:5]]}")


group_trans = np.array(group_trans, dtype=bool)
group_mono = np.array(group_mono, dtype=bool)
group_not_specified = np.array(group_not_specified, dtype=bool)
assert group_trans.shape[0] == len(ge_n)

print(f"Len ge_n: {len(ge_n)}")
print(f"Len ge_c: {len(ge_c)}")

# -------------- Histogram setup -------------- #
# WARNING: histogram boundaries are hardcoded
nbins = round((5 - (-5)) / args.step) + 1
bins = np.linspace(-5, 5, nbins, endpoint=True)
bins_centers = 0.5 * (bins[1:] + bins[:-1])

# -------------- Plots setup -------------- #
matplotlib.rcParams["font.size"] = 24
matplotlib.rcParams["text.usetex"] = True
matplotlib.rcParams["font.family"] = "modern computer"
linewidth = 1
marker = "o"
markersize = 5
colorpalette = sns.color_palette("Paired")

# -------------- Histogram and Bootstrap -------------- #
# order: 0 -> n, 1 -> c
data = [ge_n, ge_c]
labels = ["$N$-thread", "$C$-thread"]
histograms = []
ci = []
for ge_values, label in zip(data, labels):
    n, _ = np.histogram(ge_values, bins=bins, density=True)
    histograms.append(n)
    # CI
    print(f"---- Bootstrap for {label} ----")
    boot = bootstrap_hist(ge_values, bins, args.n_boots)
    original = {"statistic": n, "dataset": ge_values, "bins": nbins - 1}
    ci.append(
        statistics_ci(boot, args.n_boots, original, 0.05)
    )

# -------------- CI comparison -------------- #
# ci_file = parent / "ci_comparison.csv"
# with open(ci_file, "w") as out_file:
#     writer = csv.writer(out_file, delimiter=",")
#     writer.writerow(["# Generated by ge_c_and_n_distr.py"])
#     writer.writerow(["# ge", "hist_n", "ci_n", "hist_c", "ci_c"])
#     for ge, h_n, h_c, ci_n, ci_c in zip(bins_centers, histograms[0], histograms[1], ci[0], ci[1]):
#         ci_n_norm = ci_n / args.step / len(ge_n)
#         ci_c_norm = ci_c / args.step / len(ge_n)
#         writer.writerow(
#             [
#                 f"{ge:.3f}",
#                 f"{h_n:.5f}",
#                 f"[{ci_n_norm[0]:.5f}; {ci_n_norm[1]:.5f}]",
#                 f"{h_c:.5f}",
#                 f"[{ci_c_norm[0]:.5f}; {ci_c_norm[1]:.5f}]"
#             ]
#         )

# -------------- Plot -------------- #
fig, ax = plt.subplots(1, figsize=(10, 10))
assert len(histograms) == len(labels) == len(ci)
idx_colors = [(1, 0), (7, 6)]
for hist, label, ci_current, col in zip(histograms, labels, ci, idx_colors):
    # normalize errors
    ci_norm = ci_current / args.step / len(ge_n)
    ci_norm = np.abs(ci_norm - hist.reshape(len(hist), 1))
    ci_norm = ci_norm.T  # see plt.errorbar documentation
    ax.errorbar(
        bins_centers,
        hist,
        yerr=ci_norm,
        fmt="-o",
        color=colorpalette[col[1]],
        markersize=markersize,
        ecolor=colorpalette[col[0]],
        capsize=3,
        label=label,
        linewidth=linewidth,
    )

ax.set_xlabel(r"$\max G'$", labelpad=15)
ax.set_ylabel(r"$P\left( \max G' \right)$", labelpad=15)

remove_right_upper_axes(ax)
refined_text(ax, f"$N = {len(ge_n)}$", 0.2, 0.9)
set_ticks(ax, {"x_major": 1, "x_minor": 0.25})
fancy_grid(ax, {"y_major": True, "x_major": True})

ax.set_xlim(-3.05, 3.05)
# ax.set_ylim(-0.001, 0.055)
ax.legend(loc="upper right", fontsize=22)

fig.savefig(
    parent / "plots" / f"ge_n_c_distr_{args.region}.pdf", bbox_inches="tight"
)

# -------------- Scatter w/ trans-mono -------------- #
print("start scatter plot")
fig, ax = plt.subplots(1, figsize=(10, 10))

ge_n = np.array(ge_n)
ge_c = np.array(ge_c)

colors = [colorpalette[1], colorpalette[6], "grey"]
labels = ["Transmembrane", "Monotopic", "Not specified"]
groups = [group_trans, group_mono, group_not_specified]
for gr, la, co in zip(groups, labels, colors):
    print(f"Group {la} has {(gr == 1).sum()} elements")
    ax.scatter(
        ge_n[gr], ge_c[gr],
        color=co,
        label=la,
        alpha=0.6
    )

# draw diagonal and anti for clarity
ax.plot(
    [-2.5, 2.5], [-2.5, 2.5], color="black", linestyle="--",
    linewidth=1, alpha=0.6, zorder=0
)
ax.plot(
    [-2.5, 2.5], [2.5, -2.5], color="black", linestyle="--",
    linewidth=1, alpha=0.6, zorder=0
)

ax.set_xlabel(r"$G'_n$", labelpad=15)
ax.set_ylabel(r"$G'_c$", labelpad=15)

remove_right_upper_axes(ax)
set_ticks(ax, {"x_major": 1, "x_minor": 0.25, "y_major": 1, "y_minor": 0.25})
fancy_grid(ax, {"y_major": True, "x_major": True})

ax.legend(loc="upper left", fontsize=20)

fig.savefig(
    parent / "plots" / f"ge_n_c_trans-mono_scatter_{args.region}.pdf", bbox_inches="tight"
)
