"""Compute G'_c and G'_n for a given domains."""
import argparse
import csv
import pathlib
import sys

from pyge.gent import GE

parent = pathlib.Path(__file__).parent.resolve()
sys.path.append(str(parent / ".."))

from stats_dataset.utils import (fancy_grid, refined_text,
                                 remove_right_upper_axes, set_ticks)


# -------------- Internal function -------------- #
def ge_configuration(ge_list_complete, loop_min_len):
    # Maximum
    fun_selection = max
    params = {"key": abs}

    ge_loop_filtered = [
        x for x in ge_list_complete if x.loop[1] - x.loop[0] >= loop_min_len
    ]
    if len(ge_loop_filtered) == 0:
        return None
    ge_values = [ge.value for ge in ge_loop_filtered]

    ge_selected = fun_selection(ge_values, **params)
    loop = ge_loop_filtered[ge_values.index(ge_selected)].loop
    thread = ge_loop_filtered[ge_values.index(ge_selected)].thread

    return GE(loop=loop, thread=thread, value=ge_selected)


# -------------- Parse arguments -------------- #
parser = argparse.ArgumentParser()
parser.add_argument(
    "region",
    help="Choose between 'domains' and 'chains'.",
    choices=["domains", "chains"],
)
args = parser.parse_args()

# -------- load csv with domains to exclude --------- #
# those domains with multiple fragments
exclude_domains = []
excluded_fragment_file = parent / "fragmented_domains.csv"
with open(excluded_fragment_file, "r") as in_file:
    reader = csv.reader(in_file, delimiter=",")
    for row in reader:
        if row[0].startswith("#"):
            continue
        else:
            exclude_domains.append(row[0])

# -------------- Load and save G'_c and G'_n -------------- #
ge_dir = pathlib.Path("/mnt/GE_lx")
input_dir = ge_dir / args.region.upper() / "LOOPS"
file_list = [f for f in input_dir.iterdir() if f.suffix == ".csv"]

parent = pathlib.Path(__file__).parent.resolve()
out_file = parent / "ge_list_files" / f"ge_c_n_{args.region}.csv"

with open(out_file, "w") as out:
    writer = csv.writer(out, delimiter=",")
    writer.writerow(["# Generated by ge_c_n_config.py"])
    writer.writerow(
        [
            "# Domain",
            "n_loop_start", "n_loop_end",
            "n_thread_start", "n_thread_end", "ge_n",
            "c_loop_start", "c_loop_end",
            "c_thread_start", "c_thread_end", "ge_c"
        ]
    )

    for file in file_list:
        # skip unwanted domains
        if (file.stem[:5].upper() in exclude_domains) or (file.stem.upper() in exclude_domains):
            print(f"Excluding {file.stem}")
            continue

        # Load data
        ge_c = []
        ge_n = []
        with open(file, "r") as in_file:
            reader = csv.reader(in_file, delimiter=",")
            for row in reader:
                if row[0].startswith("#"):
                    continue
                else:
                    ge_n.append(
                        GE(
                            loop=(int(row[0]), int(row[1])),
                            thread=(int(row[3]), int(row[4])),
                            value=float(row[2])
                        )
                    )
                    ge_c.append(
                        GE(
                            loop=(int(row[0]), int(row[1])),
                            thread=(int(row[6]), int(row[7])),
                            value=float(row[5])
                        )
                    )
        # Compute G'_c and G'_n for the whole domain
        ge_c_whole = ge_configuration(ge_c, 0)
        ge_n_whole = ge_configuration(ge_n, 0)

        if abs(ge_n_whole.value) > 1 and abs(ge_c_whole.value) > 1:
            print(file.stem)
            print(f"ge_n_whole: {ge_n_whole}")
            print(f"ge_c_whole: {ge_c_whole}")

        # write result
        if ge_n_whole is None:
            if ge_c_whole is None:
                writer.writerow(
                    [
                        file.stem,
                        "None",
                        "None",
                        "None",
                        "None",
                        "None",
                        "None",
                        "None",
                        "None",
                        "None",
                        "None",
                    ]
                )
            else:
                writer.writerow(
                    [
                        file.stem,
                        "None",
                        "None",
                        "None",
                        "None",
                        "None",
                        ge_c_whole.loop[0],
                        ge_c_whole.loop[1],
                        ge_c_whole.thread[0],
                        ge_c_whole.thread[1],
                        f"{ge_c_whole.value:.5f}",
                    ]
                )
        elif ge_c_whole is None:
            writer.writerow(
                [
                    file.stem,
                    ge_n_whole.loop[0],
                    ge_n_whole.loop[1],
                    ge_n_whole.thread[0],
                    ge_n_whole.thread[1],
                    f"{ge_n_whole.value:.5f}",
                    "None",
                    "None",
                    "None",
                    "None",
                    "None",
                ]
            )
        else:
            writer.writerow(
                [
                    file.stem,
                    ge_n_whole.loop[0],
                    ge_n_whole.loop[1],
                    ge_n_whole.thread[0],
                    ge_n_whole.thread[1],
                    f"{ge_n_whole.value:.5f}",
                    ge_c_whole.loop[0],
                    ge_c_whole.loop[1],
                    ge_c_whole.thread[0],
                    ge_c_whole.thread[1],
                    f"{ge_c_whole.value:.5f}",
                ]
            )
