"""Distribution for Max G'

Generates distributions for max G', for both membrane proteins
and globular ones, and for each species.
"""
import argparse
import csv
import json
import pathlib

import matplotlib
import matplotlib.pyplot as plt
import numpy as np
import seaborn as sns
# from bootstrap import bootstrap_hist, statistics_ci

from utils import (domain_segments_parser, fancy_grid,
                                 refined_text, remove_right_upper_axes,
                                 set_ticks)


parent = pathlib.Path(__file__).parent.resolve()

# -------------- Parse arguments -------------- #
parser = argparse.ArgumentParser()
parser.add_argument(
    "region",
    help="Choose between 'domains' and 'chains'.",
    choices=["domains", "chains"],
)
parser.add_argument(
    "step",
    help="Bins fixed width.",
    type=float,
)
args = parser.parse_args()

# -------- load csv with domains to exclude --------- #
# those domains with multiple fragments
exclude_domains = []
excluded_fragment_file = parent / "fragmented_domains.csv"
with open(excluded_fragment_file, "r") as in_file:
    reader = csv.reader(in_file, delimiter=",")
    for row in reader:
        if row[0].startswith("#"):
            continue
        else:
            exclude_domains.append(row[0])

# -------- to collect domain boundaries info --------- #
membrane_dir = pathlib.Path("/mnt/MEMBRANE")
with open(membrane_dir / "chains_list.csv", "r") as file_cath:
    reader = csv.reader(file_cath)
    domains_bd = {}
    for row in reader:
        domains = domain_segments_parser(
            json.loads(row[4].replace("'", "\""))
        )
        dom_temp = {}
        for dom, bd in domains.items():
            dom_temp[dom.upper()] = bd
        domains_bd.update(dom_temp)

# -------------- Load G' PDF for globular -------------- #
globular_file = parent / ".." / "survivor_globular_trans_mono" / "hist_G1C_all.dat"

with open(globular_file, "r") as in_file:
    reader = csv.reader(in_file, delimiter=",")
    bins_centers_all = []
    ge_globular = []
    for row in reader:
        if not row[0].startswith("#"):
            split = row[0].split(" ")
            bins_centers_all.append(float(split[0]))
            ge_globular.append(float(split[5]))

# -------------- Load species data -------------- #
ge_dir = pathlib.Path("/mnt/GE_lx")
input_dir = ge_dir / args.region.upper() / "CONFIG"
species_dir = parent / ".." / "pipeline/3_classification/classification/species_list.csv"

with open(species_dir, "r") as in_file:
    reader = csv.reader(in_file, delimiter=",")
    species = {}
    next(reader)
    for row in reader:
        if not row[0] in species:
            species[row[0].replace("_", "").upper()] = row[1]

# -------------- Load results -------------- #
# Save a file too
ge_max_file = parent / "ge_list_files" / f"maxge_{args.region}.csv"

classification = []
with open(ge_max_file, "w") as out_file:
    ge_max_writer = csv.writer(out_file)
    ge_max_writer.writerow(["# Generated with ge_distr.py"])
    ge_max_writer.writerow(["# domain", "max_ge", "loop", "thread", "domain"])

    file_list = [f for f in input_dir.iterdir() if f.suffix == ".csv"]
    ge_max = []
    ge_w = []
    counter_ge_1 = 0
    for file in file_list:
        if (file.stem[:5].upper() in exclude_domains) or (file.stem.upper() in exclude_domains):
            print(f"Excluding {file.stem}")
            continue

        with open(file, "r") as in_file:
            reader = csv.reader(in_file, delimiter=",")
            for row in reader:
                if row[0].startswith("#"):
                    continue
                else:
                    # DATA EXTRACTION
                    chain_id = file.stem[:5]
                    if chain_id not in species:
                        print("NOT CLASSIFIED: ", chain_id)
                        continue
                    
                    ge_max.append(float(row[0]))
                    ge_w.append(float(row[-1]))
                    if "homo sapiens" in species[chain_id].lower():
                        classification.append(0)
                    elif "escherichia coli" in species[chain_id].lower():
                        classification.append(1)
                    elif "bos taurus" in species[chain_id].lower():
                        classification.append(2)
                    elif "mus musculus" in species[chain_id].lower():
                        classification.append(3)
                    # elif "rattus norvegicus" in species[chain_id].lower():
                    #     classification.append(4)
                    elif "saccharomyces cerevisiae" in species[chain_id].lower():
                        classification.append(4)
                    else:
                        classification.append(5)

                    # save list
                    ge_max_writer.writerow(
                        [
                            file.stem,
                            row[0], f"({row[1]}, {row[2]})", f"({row[3]}, {row[4]})",
                            f"{domains_bd[file.stem][0]}"
                        ]
                    )

                # stats
                if abs(float(row[0])) >= 1.0:
                    counter_ge_1 += 1
                # if float(row[0]) > 0 and float(row[0]) < 0.22:
                #     print(f"{file.stem} {row[0]} {domains_bd[file.stem]}")

print(f"Fraction of domains with |max G'| > 1: {counter_ge_1/len(ge_max)}")

# convert classification
ge_max_class = [
    [],
    [],
    [],
    [],
    [],
    []
]
for ge, cl in zip(ge_max, classification):
    if cl == 0:
        ge_max_class[0].append(ge)
    elif cl == 1:
        ge_max_class[1].append(ge)
    elif cl == 2:
        ge_max_class[2].append(ge)
    elif cl == 3:
        ge_max_class[3].append(ge)
    elif cl == 4:
        ge_max_class[4].append(ge)
    elif cl == 5:
        ge_max_class[5].append(ge)
    else:
        print("ERROR in ge max class")

# -------------- Histogram setup -------------- #
# WARNING: histogram boundaries are hardcoded
nbins = round((5 - (-5)) / args.step) + 1
bins = np.linspace(-5, 5, nbins, endpoint=True)
bins_centers = 0.5 * (bins[1:] + bins[:-1])

# -------------- Plots setup -------------- #
matplotlib.rcParams["font.size"] = 24
matplotlib.rcParams["text.usetex"] = True
matplotlib.rcParams["font.family"] = "modern computer"
linewidth = 5
marker = "o"
markersize = 8
colorpalette = sns.color_palette("Paired")

# -------------- Max G' -------------- #
# Blobular + membrane ::: species
fig = plt.figure(figsize=(22, 13))
subfigs = fig.subfigures(1, 2)

# Total distribution
axs0 = subfigs[0].subplots(1)
# Membrane
density, _, _ = axs0.hist(
    ge_max,
    bins=bins,
    density=True,
    color=colorpalette[0],
    rwidth=0.9,
    zorder=10,
    label="Membrane"
)
# Globular
axs0.plot(
    bins_centers_all, ge_globular,
    'o-',
    color=colorpalette[1],
    linewidth=linewidth,
    markersize=markersize,
    zorder=10,
    label="Reference"
)

axs0.set_xlabel(r"$G'_{\mathrm{max}}$", labelpad=15)
axs0.set_ylabel(r"$P\left( G'_{\mathrm{max}} \right)$", labelpad=15)

remove_right_upper_axes(axs0)
refined_text(axs0, f"$N_m = {len(ge_max)}$\n$N_r = 16709$", 0.85, 0.85)
set_ticks(axs0, {"x_major": 1, "x_minor": 0.5})
fancy_grid(axs0, {"y_major": True, "x_major": True})

axs0.set_xlim(-3.05, 3.05)
axs0.legend(loc="upper left")

# species
axs = subfigs[1].subplots(
    len(ge_max_class) - 1,
    sharex=True
)
species2num = {
    "Homo Sapiens": 0,
    "Escherichia Coli": 1,
    "Bos Taurus": 2,
    "Mus Musculus": 3,
    # "Rattus Norvegicus": 4,
    "Saccharomyces cerevisiae": 4,
    "Other": 5
}
species_labels = [
    "Homo Sapiens",
    "Escherichia Coli",
    "Bos Taurus",
    "Mus Musculus",
    # "Rattus Norvegicus",
    "Saccharomyces cerevisiae",
    "Other"
]
colorpalette = sns.color_palette()
colors = [
    colorpalette[0],
    colorpalette[1],
    colorpalette[2],
    colorpalette[3],
    colorpalette[4],
    colorpalette[9]
]
for idx, (ge_c, ax) in enumerate(zip(ge_max_class, axs.flatten())):
    if idx == 5: continue  # skip others
    ax.hist(
        ge_c,
        bins=bins,
        density=True,
        color=colors[idx],
        zorder=10
    )

    if idx == len(ge_max_class) - 2:
        ax.set_xlabel(r"$G'_{\mathrm{max}}$", labelpad=15)
    ax.yaxis.set_label_position("right")
    ax.set_ylabel(
        species_labels[idx],
        rotation=0,
        labelpad=25,
        ha="left"
    )

    remove_right_upper_axes(ax)
    ax.text(
        s=f"$N = {len(ge_c)}$",
        x=0.9, y=0.85, transform=ax.transAxes,
        ha='center', va='center',
        fontsize=18,
        bbox={
            'facecolor': 'white', 'pad': 10, 'edgecolor': '#B2B2B2',
        }
    )
    set_ticks(ax, {"x_major": 1, "x_minor": 0.5})
    fancy_grid(ax, {"y_major": True, "x_major": True})

    ax.set_xlim(-3.05, 3.05)


fig.savefig(
    parent / "plots" / f"g_max_total_species_{args.region}.pdf",
    bbox_inches="tight"
)

# save a file with histogram data
with open(parent / "distribution_files" / f"maxge_{args.region}.csv", "w") as out_file:
    n, _ = np.histogram(ge_max, bins=bins)
    writer = csv.writer(out_file)
    writer.writerow(["# Generated with ge_distr.py"])
    writer.writerow([f"# bin width: {args.step:.4f}"])
    writer.writerow(["# bin_center", "frequency", "counts"])
    for i, (freq, counts) in enumerate(zip(density, n)):
        writer.writerow([f"{bins_centers[i]:.6}", f"{freq:.6f}", f"{counts:d}"])

# -------------- Scatter -------------- #
fig, ax = plt.subplots(1, 1, figsize=(11, 11))

ax.plot(
    ge_max, ge_w,
    marker="o", markersize=5,
    linestyle="None",
    color=sns.color_palette("Paired")[5]
)
ax.set_xlabel("$\\max G'$")
ax.set_ylabel("$\\langle G' \\rangle$")

remove_right_upper_axes(ax)
set_ticks(ax, {"x_major": 1, "x_minor": 0.5, "y_major": 0.5, "y_minor": 0.25})
fancy_grid(ax, {"y_major": True, "x_major": True})

fig.savefig(
    parent / "plots" / f"ge_scatter_{args.region}.pdf",
    bbox_inches="tight"
)
