"""TRANS/MONO plots.

The scripts generates the image containing:
- trans/mono distribution
- cumulative distributions for all, trans and mono
- domain len distributions for all, trans and mono
"""
import argparse
import csv
import json
import pathlib

import matplotlib
import matplotlib.pyplot as plt
import numpy as np
import seaborn as sns

from utils import (domain_segments_parser, fancy_grid, refined_text,
                   remove_right_upper_axes, set_ticks)

# from bootstrap import bootstrap_hist, statistics_ci


parent = pathlib.Path(__file__).parent.resolve()

# -------------- Parse arguments -------------- #
parser = argparse.ArgumentParser()
parser.add_argument(
    "region",
    help="Choose between 'domains' and 'chains'.",
    choices=["domains", "chains"],
)
parser.add_argument(
    "step",
    help="Bins step to use for the histogram.",
    type=float,
)
args = parser.parse_args()

# -------- load csv with domains to exclude --------- #
# those domains with multiple fragments
exclude_domains = []
excluded_fragment_file = parent / "fragmented_domains.csv"
with open(excluded_fragment_file, "r") as in_file:
    reader = csv.reader(in_file, delimiter=",")
    for row in reader:
        if row[0].startswith("#"):
            continue
        else:
            exclude_domains.append(row[0])

# -------------- Load data -------------- #
ge_dir = pathlib.Path("/mnt/GE_lx")
input_dir = ge_dir / args.region.upper() / "CONFIG"
trans_mono_dir = parent / ".." / "pipeline/3_classification/classification/transmembrane_monotopic_list.csv"

# parse trans-mono info
with open(trans_mono_dir, "r") as in_file:
    reader = csv.reader(in_file, delimiter=",")
    transmono = {}
    next(reader)
    for row in reader:
        if not row[0] in transmono:
            transmono[row[0].replace("_", "").upper()] = row[1]

file_list = [f for f in input_dir.iterdir() if f.suffix == ".csv"]
ge_max = []
ge_w = []
ge_max_trans = []
ge_max_mono = []
classification = []
for file in file_list:
    if (file.stem[:5].upper() in exclude_domains) or (file.stem.upper() in exclude_domains):
        print(f"Excluding {file.stem}")
        continue
        
    with open(file, "r") as in_file:
        reader = csv.reader(in_file, delimiter=",")
        for row in reader:
            if row[0].startswith("#"):
                continue
            else:
                chain_id = file.stem[:5]
                if chain_id not in transmono:
                    print("NOT CLASSIFIED: ", chain_id)
                    continue
                ge_max.append(float(row[0]))
                ge_w.append(float(row[-1]))
                if transmono[chain_id] == "mono":
                    ge_max_mono.append(float(row[0]))
                    classification.append("mono")
                elif transmono[chain_id] == "trans":
                    ge_max_trans.append(float(row[0]))
                    classification.append("trans")

# -------------- load survivor -------------- #
g_survivor = []

all_survivor = []
all_survivor_err_up = []
all_survivor_err_low = []

trans_survivor = []
trans_survivor_err_up = []
trans_survivor_err_low = []

mono_survivor = []
mono_survivor_err_up = []
mono_survivor_err_low = []

all_survivor_file = parent / ".." / "survivor_globular_trans_mono" / "classes_G1C_all_bootstrap_1e4.dat"
trans_survivor_file = parent / ".." / "survivor_globular_trans_mono" / "classes_G1C_trans_membrane_bootstrap_1e4.dat"
mono_survivor_file = parent / ".." / "survivor_globular_trans_mono" / "classes_G1C_mono_membrane_bootstrap_1e4.dat"

# all
with open(all_survivor_file, "r") as in_file:
    reader = csv.reader(in_file, delimiter=" ")
    for row in reader:
        g_survivor.append(float(row[0]))
        all_survivor_err_up.append(float(row[1]))
        all_survivor.append(float(row[2]))
        all_survivor_err_low.append(float(row[3]))
all_survivor = np.array(all_survivor)
all_survivor_err_up = np.array(all_survivor_err_up)
all_survivor_err_low = np.array(all_survivor_err_low)
all_survivor_err_up = all_survivor_err_up - all_survivor
all_survivor_err_low = all_survivor - all_survivor_err_low
# trans
with open(trans_survivor_file, "r") as in_file:
    reader = csv.reader(in_file, delimiter=" ")
    for row in reader:
        trans_survivor_err_up.append(float(row[1]))
        trans_survivor.append(float(row[2]))
        trans_survivor_err_low.append(float(row[3]))
trans_survivor = np.array(trans_survivor)
trans_survivor_err_up = np.array(trans_survivor_err_up)
trans_survivor_err_low = np.array(trans_survivor_err_low)
trans_survivor_err_up = trans_survivor_err_up - trans_survivor
trans_survivor_err_low = trans_survivor - trans_survivor_err_low
# mono
with open(mono_survivor_file, "r") as in_file:
    reader = csv.reader(in_file, delimiter=" ")
    for row in reader:
        mono_survivor_err_up.append(float(row[1]))
        mono_survivor.append(float(row[2]))
        mono_survivor_err_low.append(float(row[3]))
mono_survivor = np.array(mono_survivor)
mono_survivor_err_up = np.array(mono_survivor_err_up)
mono_survivor_err_low = np.array(mono_survivor_err_low)
mono_survivor_err_up = mono_survivor_err_up - mono_survivor
mono_survivor_err_low = mono_survivor - mono_survivor_err_low

# -------------- load len distr for all -------------- #
all_len = []
all_len_file = all_survivor_file = parent / ".." / "survivor_globular_trans_mono" / "G1C_prot_data_revised.dat"

with open(all_len_file, "r") as in_file:
    reader = csv.reader(in_file, delimiter="\t")
    for row in reader:
        all_len.append(int(row[3]))

# -------------- compute len distr for mono/trans -------------- #
list_domains_with_ge = [f.stem for f in file_list]
with open(trans_mono_dir, "r") as data_file:
    reader = csv.reader(data_file)
    next(reader)
    mono_len = []
    trans_len = []
    for row in reader:
        domains = domain_segments_parser(json.loads(row[2].replace("'", '"')))
        for dom, segs in domains.items():
            if dom.upper() in exclude_domains:
                continue
            if not dom.upper() in list_domains_with_ge:
                continue
            if len(segs) > 1:
                raise RuntimeError("More than one segment per domain")
            dom_len = segs[0][1] - segs[0][0] + 1

            if row[1] == "mono":
                mono_len.append(dom_len)
            elif row[1] == "trans":
                trans_len.append(dom_len)
            elif row[1] == "other":
                continue
            else:
                raise RuntimeError("Unknown type")

# -------------- Histogram setup -------------- #
# WARNING: histogram boundaries are hardcoded
nbins = round((5 - (-5)) / args.step) + 1
bins = np.linspace(-5, 5, nbins, endpoint=True)
bins_centers = 0.5 * (bins[1:] + bins[:-1])

# -------------- Plots setup -------------- #
matplotlib.rcParams["font.size"] = 24
matplotlib.rcParams["text.usetex"] = True
matplotlib.rcParams["font.family"] = "modern computer"
linewidth = 2
marker = "o"
markersize = 5
colorpalette = sns.color_palette("Paired")

fig = plt.figure(figsize=(22, 14))

# -------------- Histogram -------------- #
subfigs = fig.subfigures(1, 2)
axs0 = subfigs[0].subplots(1)

density_t, _, _ = axs0.hist(
    ge_max_trans,
    bins=bins,
    density=True,
    color=colorpalette[0],
    label=f"Transmembrane $= {len(ge_max_trans)}$",
    zorder=10,
    fill=False,
    histtype='step',
    edgecolor=colorpalette[1], linewidth=3
)
density_m, _, _ = axs0.hist(
    ge_max_mono,
    bins=bins,
    density=True,
    color=colorpalette[6],
    label=f"Monotopic $= {len(ge_max_mono)}$",
    zorder=5,
    fill=False,
    histtype='step',
    edgecolor=colorpalette[7], linewidth=3
)

axs0.set_xlabel(r"$G'_{\mathrm{max}}$", labelpad=15)
axs0.set_ylabel(r"$P\left( G'_{\mathrm{max}} \right)$", labelpad=15)

remove_right_upper_axes(axs0)
# refined_text(ax, f"$N = {len(ge_max)}$", 0.8, 0.9)
set_ticks(axs0, {"x_major": 1, "x_minor": 0.5})
fancy_grid(axs0, {"y_major": True, "x_major": True})

axs0.set_xlim(-3.05, 3.05)
axs0.legend(fontsize=20)

# save a file with histogram data
with open(parent / "distribution_files" / f"maxge_trans-mono_{args.region}.csv", "w") as out_file:
    n_t, _ = np.histogram(ge_max_trans, bins=bins)
    n_m, _ = np.histogram(ge_max_mono, bins=bins)
    writer = csv.writer(out_file)
    writer.writerow(["# Generated with ge_distr.py"])
    writer.writerow([f"# bin width: {args.step:.4f}"])
    writer.writerow(["# bin_center", "frequency_trans", "frequency_mono", "counts_trans", "counts_mono"])
    for i, (freq_t, freq_m, counts_t, counts_m) in enumerate(zip(density_t, density_m, n_t, n_m)):
        writer.writerow([f"{bins_centers[i]:.6}", f"{freq_t:.6f}", f"{freq_m:.6f}", f"{counts_t:d}", f"{counts_m:d}"])


# -------------- survivor and length distribution -------------- #
axs = subfigs[1].subplots(2)

# survivor
axs[0].errorbar(
    g_survivor,
    all_survivor,
    yerr=[all_survivor_err_low, all_survivor_err_up],
    fmt="-o",
    color="grey",
    markersize=markersize,
    ecolor="black",
    capsize=3,
    label="Ref",
    linewidth=linewidth,
)
axs[0].errorbar(
    g_survivor[:-7],
    trans_survivor,
    yerr=[trans_survivor_err_low, trans_survivor_err_up],
    fmt="-o",
    color=colorpalette[0],
    markersize=markersize,
    ecolor=colorpalette[1],
    capsize=3,
    label="Trans",
    linewidth=linewidth,
)
axs[0].errorbar(
    g_survivor[:-7],
    mono_survivor,
    yerr=[mono_survivor_err_low, mono_survivor_err_up],
    fmt="-o",
    color=colorpalette[6],
    markersize=markersize,
    ecolor=colorpalette[7],
    capsize=3,
    label="Mono",
    linewidth=linewidth,
)

axs[0].set_xlabel(r"$|G'_{\mathrm{max}}|$", labelpad=15)
axs[0].set_ylabel("Survivor", labelpad=15)

remove_right_upper_axes(axs[0])
set_ticks(axs[0], {"x_major": 0.5, "x_minor": 0.25})
fancy_grid(axs[0], {"y_major": True, "x_major": True})

axs[0].set_xlim(-0.02, 2.05)
axs[0].legend()

# Len
bins = np.arange(0, 801, 20)
axs[1].hist(
    all_len,
    bins=bins,
    density=True,
    color="grey",
    label="Ref",
    zorder=1,
    # alpha=0.4,
    fill=False,
    histtype = 'step',
    edgecolor="black", linewidth=1.2
)
axs[1].hist(
    trans_len,
    bins=bins,
    density=True,
    color=colorpalette[0],
    label="Trans",
    zorder=10,
    # alpha=0.4,
    fill=False,
    histtype = 'step',
    edgecolor=colorpalette[1], linewidth=1.2
)
axs[1].hist(
    mono_len,
    bins=bins,
    density=True,
    color=colorpalette[6],
    label="Mono",
    zorder=5,
    # alpha=0.4,
    fill=False,
    histtype = 'step',
    edgecolor=colorpalette[7], linewidth=1.2
)

axs[1].set_xlabel("Domain length", labelpad=15)
axs[1].set_ylabel("Distribution", labelpad=15)

remove_right_upper_axes(axs[1])
set_ticks(axs[1], {"x_major": 200, "x_minor": 50})
fancy_grid(axs[1], {"y_major": True, "x_major": True})

# axs[1].set_xlim(-3.05, 3.05)
axs[1].legend(fontsize=20)

# save final plot
fig.savefig(
    parent / "plots" / f"g_max_survivor_len_{args.region}.pdf",
    bbox_inches="tight"
)
