"""Draw grouping fraction plots."""
import argparse
import csv
import json
import pathlib

import matplotlib
import matplotlib.colors
import matplotlib.pyplot as plt
import numpy as np
import seaborn as sns

from utils import (domain_segments_parser, fancy_grid, refined_text,
                   remove_right_upper_axes, set_ticks)


parent = pathlib.Path(__file__).parent.resolve()


# -------------- load survivor -------------- #
def survivor(file, indexes):
    """Load survivor data."""
    g_survivor = []
    survivor = []
    survivor_err_up = []
    survivor_err_low = []
    with open(file, "r") as in_file:
        reader = csv.reader(in_file, delimiter=" ")
        for row in reader:
            g_survivor.append(float(row[indexes[0]]))
            survivor_err_up.append(float(row[indexes[1]]))
            survivor.append(float(row[indexes[2]]))
            survivor_err_low.append(float(row[indexes[3]]))
    survivor = np.array(survivor)
    survivor_err_up = np.array(survivor_err_up)
    survivor_err_low = np.array(survivor_err_low)
    survivor_err_up = survivor_err_up - survivor
    survivor_err_low = survivor - survivor_err_low
    return g_survivor, survivor, survivor_err_up, survivor_err_low


# -------------- Plot specifications -------------- #
all_survivor_file = parent / ".." / "survivor_globular_trans_mono" / "classes_G1C_all_bootstrap_1e4.dat"
trans_survivor_file = parent / ".." / "survivor_globular_trans_mono" / "classes_G1C_trans_membrane_bootstrap_1e4.dat"
mono_survivor_file = parent / ".." / "survivor_globular_trans_mono" / "classes_G1C_mono_membrane_bootstrap_1e4.dat"

files = [all_survivor_file, trans_survivor_file, mono_survivor_file]
titles = ["Reference", "Transmembrane", "Monotopic"]
indexes = [
    [0, 4, 5, 6], [0, 7, 8, 9], [0, 22, 23, 24], [0, 25, 26, 27]
]
colors = [
    "#535353", "#535353", "#ff5e5e", "#ff5e5e"
]
colors_errors = [
    matplotlib.colors.to_rgba("black", 0.3),
    matplotlib.colors.to_rgba("black", 0.3),
    matplotlib.colors.to_rgba("red", 0.3),
    matplotlib.colors.to_rgba("red", 0.3)
]
markers = [
    "o-", "o--", "o-", "o--"
]
labels = [
    "N-thread $+$", "C-thread $+$", "N-thread $-$", "C-thread $-$"
]


# -------------- Plots setup -------------- #
matplotlib.rcParams["font.size"] = 24
matplotlib.rcParams["text.usetex"] = True
matplotlib.rcParams["font.family"] = "modern computer"
linewidth = 2
markersize = 4

# -------------- survivor -------------- #
fig, axs = plt.subplots(
    1, 3,
    figsize=(26, 8),
    sharey=True,
    gridspec_kw={"wspace": 0.075}
)

axs[0].set_ylabel("Grouping fraction", labelpad=20)
for ax, f, tit in zip(axs.flatten(), files, titles):

    for indx, c, c_err, mark, lab in zip(indexes, colors, colors_errors, markers, labels):
        g, s, s_err_up, s_err_low = survivor(f, indx)

        ax.errorbar(
            g,
            s,
            yerr=[s_err_low, s_err_up],
            fmt=mark,
            color=c,
            markersize=markersize,
            ecolor=c_err,
            capsize=3,
            label=lab,
            linewidth=linewidth,
        )

    ax.set_xlabel(r"$|G'_{\mathrm{max}}|$", labelpad=15)

    remove_right_upper_axes(ax)
    refined_text(ax, tit, 0.8, 0.9)
    set_ticks(ax, {"x_major": 0.5, "x_minor": 0.25})
    fancy_grid(ax, {"y_major": True, "x_major": True})

    ax.set_ylim(-0.05, 0.75)
    if tit == 'Monotopic':
        ax.set_xlim(-0.05, 1.65)
        ax.legend(
            loc='center left',
            bbox_to_anchor=(1, 0.5)
        )
    else:
        ax.set_xlim(-0.05, 2.05)

# save final plot
fig.savefig(
    parent / "plots" / "grouping_fraction.pdf",
    bbox_inches="tight"
)
