"""Collection of function to setup plots."""
import re

import matplotlib.pyplot as plt


def domain_segments_parser(domain_dictionary):
    """
    Parse the dictionary containing sequence segments of
    a domain to return another dictionary easier to handle
    """
    out = {}
    for domain, segment in domain_dictionary.items():
        seq_segs = segment.split(",")
        segs = []
        for seq in seq_segs:
            start, end = seq.split("-")
            segs.append([int(start), int(end)])
        out[domain] = segs
    return out


def remove_right_upper_axes(axes):
    """Pretty trivial"""
    axes.spines.right.set_visible(False)
    axes.spines.top.set_visible(False)


def refined_text(axes, text, x, y):
    """Refined text box for plots"""
    axes.text(
        s=text,
        x=x,
        y=y,
        transform=axes.transAxes,
        ha="center",
        va="center",
        bbox={"facecolor": "white", "pad": 10},
    )


def set_ticks(axes, tick_params):
    """
    Set up the tick positions using MultipleLocator for both x and y axes.

    Parameters:
        ax (matplotlib.axes.Axes): the axes to modify.
        tick_params (dict): a dictionary of tick parameters, with the following keys:
            - x_major: the MultipleLocator for the x-axis major ticks.
            - x_minor: the MultipleLocator for the x-axis minor ticks.
            - y_major: the MultipleLocator for the y-axis major ticks.
            - y_minor: the MultipleLocator for the y-axis minor ticks.
    """
    x_major = tick_params.get("x_major", None)
    x_minor = tick_params.get("x_minor", None)
    y_major = tick_params.get("y_major", None)
    y_minor = tick_params.get("y_minor", None)

    if x_major is not None:
        axes.xaxis.set_major_locator(plt.MultipleLocator(x_major))
    if x_minor is not None:
        axes.xaxis.set_minor_locator(plt.MultipleLocator(x_minor))
    if y_major is not None:
        axes.yaxis.set_major_locator(plt.MultipleLocator(y_major))
    if y_minor is not None:
        axes.yaxis.set_minor_locator(plt.MultipleLocator(y_minor))


def fancy_grid(axes, grid_params):
    """
    Modify the grid settings for a given set of axes using a dictionary of grid parameters.
    """
    x_major = grid_params.get("x_major", False)
    x_minor = grid_params.get("x_minor", False)
    y_major = grid_params.get("y_major", False)
    y_minor = grid_params.get("y_minor", False)

    if x_major:
        axes.grid(
            which="major",
            axis="x",
            linestyle="-",
            linewidth="0.5",
            color="gray",
            alpha=0.5,
        )
    if x_minor:
        axes.grid(
            which="minor",
            axis="x",
            linestyle="-",
            linewidth="0.5",
            color="gray",
            alpha=0.2,
        )
    if y_major:
        axes.grid(
            which="major",
            axis="y",
            linestyle="-",
            linewidth="0.5",
            color="gray",
            alpha=0.5,
        )
    if y_minor:
        axes.grid(
            which="minor",
            axis="y",
            linestyle="-",
            linewidth="0.5",
            color="gray",
            alpha=0.2,
        )


def check_for_word(word, string):
    """
    Search for a word in string

    Parameters
    ----------
    word : str
        word to search
    string : str
        string where to search word

    Returns
    -------
    bool
    """
    match = re.search(word, re.escape(string), re.IGNORECASE)
    return bool(match)
